;####################################################################
; Modul fuer IDE-Karte
;
; Entwickelt aus IDETOOL von Alfred Klichs 2022,;
; Paul's 8051 Code Library, Paul Stoffregen 2005
; https://www.pjrc.com/tech/8051/ide/
;
;
; Assembler 8085-Simulator-ID V4.93 Oshonsoft (Vladimir Soso)
; Editor 	Notepad++ TAB=6
;
;
; Stand:
;	12.02.2023	Aufgeraeumt
;	24.01.2023	uebernommen aus FDC-Routinen vom 23.01.2023
;
;--------------------------------------------------------------------
;
; IO-Adressen fuer PIO 8255
;
PIO_IDE		EQU	058H		; PIO I/O Adresse
;
PIO_LSB		EQU	0 + PIO_IDE ; Port A
PIO_MSB		EQU	1 + PIO_IDE ; Port B
PIO_CTL		EQU	2 + PIO_IDE ; Port C
PIO_CFG		EQU	3 + PIO_IDE ; I/O modus
;
PIO_RDW		EQU	10010010B 	; Port A+B input, Port C output
PIO_WR		EQU	10000000B 	; Port A+B+C output
;
IDE_DRV		EQU	11100000B 	; select the device als LBA Master
			; x1x00000b LBA master
			; x1x10000b LBA slave
			; x0x00000b CHS master
			; x0x10000b CHS slave
;			; Kanal B als Ausgabe
;
;--------------------------------------------------------------------
;
; Variable fuer IDE Funktionen
;
IDE_A0		EQU	00000001B 	; A0 direct to IDE HDD
IDE_A1		EQU	00000010B 	; A1 dto.
IDE_A2		EQU	00000100B 	; A2 dto.
IDE_CS0		EQU	00001000B 	; CS0 to Inverter
IDE_CS1		EQU	00010000B 	; CS1 dto.
IDE_WR		EQU	00100000B 	; WR dto.
IDE_RD		EQU	01000000B 	; RD dto.
IDE_RST		EQU	10000000B 	; RST dto.
;
IDE_DATA		EQU	IDE_CS0
IDE_ERR		EQU	IDE_CS0 + IDE_A0
IDE_SEC_CNT		EQU	IDE_CS0 + IDE_A1
IDE_SECTOR		EQU	IDE_CS0 + IDE_A1 + IDE_A0
IDE_CYL_LSB		EQU	IDE_CS0 + IDE_A2
IDE_CYL_MSB		EQU	IDE_CS0 + IDE_A2 + IDE_A0
IDE_HEAD		EQU	IDE_CS0 + IDE_A2 + IDE_A1
IDE_COMMAND		EQU	IDE_CS0 + IDE_A2 + IDE_A1 + IDE_A0
IDE_STATUS		EQU	IDE_CS0 + IDE_A2 + IDE_A1 + IDE_A0
IDE_CONTROL		EQU	IDE_CS1 + IDE_A2 + IDE_A1
IDE_ASTATUS		EQU	IDE_CS1 + IDE_A2 + IDE_A1 + IDE_A0
;
IDE_CMD_RECAL	EQU	010H
IDE_CMD_READ	EQU	020H
IDE_CMD_WRITE	EQU	030H
IDE_CMD_INIT	EQU	091H
IDE_CMD_ID		EQU	0ECH
IDE_CMD_SPINDN	EQU	0E0H
IDE_CMD_SPINUP	EQU	0E1H
;
;===========================================================
;
; IDE-karte lesen/schreiben
;
; I:
;	datbuf	= aktuelle Bufferadresse
;	trknr		= laufende Spurnummer
;	secnr		= laufende Sektornummer
;
;	LBA	= Sektor
;	LBA+1	= Track lsb
;	LBA+2	= Track msb
;	LBA+3	= Head
;
; O:
;	Buffer 	= 1KByte Daten, werden dann Recordweise mit DMA ausgetauscht
;	(A)		= 0 für OK
;	(CY) 		= 1 Fehler lesen/schreiben
;
;--------------------------------------------------------------------
;
; initrw =	Buffer, Spur, Sektorregister vorbereiten
;
; CHS_LBA uebertraegt die CHS zu LBA
;
; Parameteruebergabe:
;
;	datbuf	= aktuelle Bufferadresse
;
;	LBA		= Sektor
;	LBA+1		= Track lsb
;	LBA+2		= Track msb
;	LBA+3		= Head

initrw:	xra  	a		; clear wrtflg -"Buffer ungültig"
		sta  	wrtflg	;
		di			; Interrupt sperren
		sta	lba+2
		sta	lba+3		; Head 0
		lxi	h,buffer	;
		shld	datbuf	;
		lhld	akttrk	;
		shld	trknr		;
		shld	lba+1		; Track lsb/msb
		lda	aktsec	;
		sta	secnr		;
		sta	lba		; Sektor
		lxi	h,buffer	;
		shld	datbuf	;
		ret			;
;
; get/put_cf =	Block von CF lesen/schreiben
;
get_ide:	call 	initrw	;
		call 	READ_SECTOR	;	;
		ana	a		;
		jnz	cf_err	; Fehler
		;
		call 	updsek	; 2.Sektor laden
		call	READ_SECTOR	;
		ana	a		;
		jnz	cf_err	; Fehler
		xra	a		;
		ret			; zurück zum Disk-Manager
		;
		;
put_ide:	call 	initrw	;
		call 	WRITE_SECTOR		;
		ana	a		;
		jnz	cf_err	; Fehler
		call 	updsek	;
		call	WRITE_SECTOR		;
		ana	a		;
		jnz	cf_err	; Fehler
		xra	a		;
		ret			; zurück zum Disk-Manager
;
;
; updsek/sp = Sektor und Spurnummern aktualisieren
;             und an CF-Karte senden
;
;	! maimal 32 Records pro Spur !
;
updsek:	lxi	h,buffer+512
		shld	datbuf
		lda	secnr		; Sektor holen
		cpi	16		; letzter Sektor spur
		jz	updsp		;
		inr	a		; nächster Sektor gleiche Spur
		sta	secnr		;
		sta	lba
		ret

updsp:	mvi	a,01h		; nächste Spur mit Sektor 1
		sta	secnr		;
		sta	lba
		lhld	trknr		;
		inx	h		;
		shld	trknr		;
		shld	lba+1		;
		ret
;
; Fehler (A)=1
;
cf_err:	mvi	a,01		;
		stc			;
		ret			; Zurück mit Fehler
;
;
;--------------------------------------------------------------------
;
